import os

class PathNotADirectory(Exception):
  def __init__(self, err):
    self.__err = err

  def __str__(self):
      return 'Path is not a directory: {}'.format(self.__err)

class PathNotFound(Exception):
  def __init__(self, err):
    self.__err = err

  def __str__(self):
    return 'Path not found: {}'.format(self.__err)


# Partitions generator (for partition in partitions())
def search_base(partition_base):
  if os.path.exists(partition_base):
    if os.path.isdir(partition_base):
      file_list = [os.path.abspath(partition_base + os.path.sep + filename) for filename in os.listdir(partition_base)]
      for filename in file_list:
        if not filename.endswith('lost+found'):
          yield filename
    else:
      raise PathNotADirectory(partition_base)
  else:
    raise PathNotFound(partition_base)


# Limited depth filetree search
def limited_search(path, compiled_ignores = [], depth = 1):
  max_depth = path.count(os.path.sep) + depth

  if os.path.exists(path):
    if os.path.isdir(path):

      for path, dirs, files in os.walk(path):
        current_depth = path.count(os.path.sep)

        # Remove from dirs list if we are over depth
        if current_depth >= max_depth:
          dirs[:] = []

        # Remove if we know to skip over it
        ignore_match = None
        for match in compiled_ignores:
          ignore_match = match.search(path)
          if ignore_match is not None:
            break

        if ignore_match is None:
          yield path, dirs, files

    else:
      raise PathNotADirectory(path)
  else:
    raise PathNotFound(path)
