import os
import sys
import fcntl
import atexit

class Lock:
  def __init__(self, filename):
    self.filename = filename
    # This will create it if it does not exist already
    self.handle = open(filename, 'w')

  # Bitwise OR fcntl.LOCK_NB if you need a non-blocking lock 
  def acquire(self):
    try:
      fcntl.flock(self.handle, fcntl.LOCK_EX | fcntl.LOCK_NB)
      atexit.register(self.release)
    except IOError:
      print('Requested command already in use, cannot continue')
      sys.exit(1)
    print('Acquired lock')

  def release(self):
    fcntl.flock(self.handle, fcntl.LOCK_UN)
    print('Released lock')

  def __del__(self):
    self.handle.close()


