import re
import time
import datetime

# Parse timestamp
def parse_ISO8601(date_string, raise_on_err = False):
  date_string = date_string.rstrip('\r\n')

  # Some strftime impls don't support subseconds for %S annoyingly so
  # we throw it away here even though it might be possible to parse
  # there's no easy way to tell, sigh. It also seems to do this
  # even if the impl specifically state decimal number between 0-60 with
  # no split seconds. UUrggghh
  date_string = re.sub('\.[0-9]{6}([|\+\-]{1,2})','\g<1>', date_string)

  re_timezone = re.compile('([Zz+\-]{1}[+\-]?)([0-9:]{4,5})?$')
  match_timezone = re_timezone.search(date_string)

  re_timestamp = re.compile('^([0-9]{4}-([0-9]{2}-?){2}T([0-9]{2}:?){3})([Zz+\-]{1,2}[0-9:]{4,5})?$')
  match_timestamp = re_timestamp.match(date_string)


  if match_timestamp is None:
    if raise_on_err:
      raise Exception('Failed to parse date')
    return [False, date_string]

  chars_date = list(date_string)

  if match_timezone is not None:
    if chars_date[-3] == ':':
      chars_date.pop(-3)

    if chars_date[-5] == 'Z':
      chars_date[-5] = '+'
    else:
      chars_date[-5] = '-'

    date_string = ''.join(chars_date).translate(str.maketrans({
      'Z': None,
      'z': None,
    }))

    try:
      result = [True, datetime.datetime.strptime(
        date_string,
        '%Y-%m-%dT%H:%M:%S%z')]

      if raise_on_err:
        return result[1]
      return result

    except ValueError as err:
      if raise_on_err:
        raise Exception('Failed to parse data')
      return [False, date_string]

  else:
    try:
      result = [True, datetime.datetime.strptime(
        '{}+0000'.format(''.join(chars_date)),
        '%Y-%m-%dT%H:%M:%S%z')]

      if raise_on_err:
        return result[1]
      return result

    except ValueError as err:
      if raise_on_err:
        raise Exception('Failed to parse data')
      return [False, date_string]


# ISO8601 format time
def format_ISO8601(timestamp, zfs_compat = False):
  if isinstance(timestamp, time.struct_time):
    if zfs_compat:
      time_string = timestamp.strftime('%Y-%m-%dT%H:%M:%S', timestamp)
      timezone = time.strftime('%z', timestamp)
      return '{}{}'.format(time_string, timezone.translate(str.maketrans({
        '+': 'Z',
        '-': 'z',
      })))
    else:
      return time.strftime('%Y-%m-%dT%H:%M:%S%z', timestamp)
      

  if isinstance(timestamp, datetime.datetime):
    if zfs_compat:
      time_string = timestamp.strftime('%Y-%m-%dT%H:%M:%S')
      timezone = timestamp.strftime('%z')
      return '{}{}'.format(time_string, timezone.translate(str.maketrans({
        '+': 'Z',
        '-': 'z',
      })))
    else: 
      return timestamp.isoformat()

# epoch seconds since UTC based on offset from struct_time
def epoch_utc(struct_time):
  # Hopefully mangle a struct_time to UTC seconds from an offset
  # on struct_datetime.datetime.tm_gmtoff
  offset = struct_time.tm_gmtoff

  if struct_time.tm_gmtoff is None:
    offset = 0

  # I am sure there are better ways of doing this
  epoch = time.mktime(struct_time)

  if offset > 0:
    return epoch - offset
  else:
    return epoch + offset

def sort_key_snapshot(timestamp):
  if isinstance(timestamp, time.struct_time):
    return epoch_utc(timestamp)
  elif isinstance(timestamp, datetime.datetime):
    return timestamp.timestamp()
  else:
    parsed_timestamp = parse_ISO8601(timestamp)
    if parsed_timestamp[0]:
      return parsed_timestamp[1].timestamp()
    else:
      return timestamp
